#ifndef QUEST_TYPES
#define QUEST_TYPES

#include <gtk/gtk.h>
#include <gdk_imlib.h>
#include "global.h"

typedef gint16	direction;

typedef struct Sprite {

	GdkImlibImage	*image;
	GdkPixmap	*pixmap;
	GdkBitmap	*mask;

} Sprite;

typedef struct Item {

	/* Item ID */
	GString*	name;
	gint		id;

	/* coordinates */
	gint		x;
	gint		y;
	gint		width;
	gint		height;

	/* graphical information */
	guint16		state;
	gint		sprites[ITEM_FRAMES]; /* array of sprite numbers */
	gint		sprite_cur; /* is redundant, because item.state exists,
	                               but exists to make handling items, monsters and spaces
	                               the same. */

	/* item information */
	guint16		properties;
	guint16		kind;
	gint		weight;
	
	/* battle information */
	gint		max_hp;
	gint		cur_hp;
	gint		armor; /* percentile */
	gint		tread_dmg;

	/* stacked items information */
	gint		inbag_cnt;
	gint16		inbag_mask;
	GPtrArray*	inbag;
	struct Item*	stacked; /* on the map */
	gint		where;	/* inventory location */
	struct Item*	inside;

} Item;

typedef struct Monster {
	
	/* Monster ID */
	GString*	name;
	gint		id;

	/* coordinates and planned path */
	gint		x;
	gint		y;
	gint		width;
	gint		height;
	gint		path[MONSTER_MEMORY];

	/* graphical information */
	guint16		facing;
	guint16		state;
	gint		sprites[MONSTER_FRAMES]; /* array of sprite numbers */
	gint		sprite_cur;

	/* character information */
	guint16		properties;
	guint16		race;
	guint16		access;	
	guint		agression; /* percentile */
	gint		weight;

	/* behaviour */
	void	 	(*behaviour)(); 
	GArray*		memory;
	guint		memory_cnt;

	/* battle infomation */
	gint		max_ap;
	gint		cur_ap;
	gint		max_hp;
	gint		cur_hp;
	guint		armor; /* percentile */
	GPtrArray*	attacks;
	gint		attack_cnt;
	gint		attack_cur; 
	guint		initiative; /* percentile */
	guint		initiative_adj; /* percentile */

	/* inventory */
	GPtrArray*	inventory;
	guint16		inventory_mask;

} Monster;

typedef struct Coords {

	gint 		x;
	gint 		y;

} Coords;

typedef struct Space {

	/* space information */
	gint16		type;
	gint16		variation;
	gint		sprites[SPACE_FRAMES]; /* array of sprite numbers */
	gint		sprite_cur;
	
	/* what's on the space */
	Monster*	mos;
	Item*		ios;

} Space;

typedef struct Map {

	/* Map ID */
	GString*	name;
	gint		id;

	/* the actual map */
	Space*		cell[MAX_MAP_X][MAX_MAP_Y];

} Map;

typedef struct Attack {

	gint		tohit;
	gint		min_dmg;
	gint		max_dmg;
	gint		ap_cost;

} Attack;

typedef struct MapWin {

	GtkWidget* window;
	GtkWidget* box;
	GtkWidget* menu;
	GtkWidget* drawing_area;
	GdkPixmap* area;
	GdkGC*     gc;
	gint       area_cur;

	/* the update stack, Coords */
	GList*		update;

	/* the location of the visible area on the map */
	gint16		area_ox;
	gint16		area_oy;

} MapWin;

typedef struct GameControl {

	Map*		map;
	MapWin*		map_window;
	GList*		image_files;
	GList*		sprites;
	GList*		monsters;
	GList*		items;
	gint32		turn;
	gint16		game_state;
	guint8		cycle_cnt;

} GameControl;

#endif /* QUEST_TYPES */